#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:12
@File: __init__.py
@IDE: pycharm
@Description:
    日志模块，提供swanlab标准日志记录功能
"""
from .utils import FONT
from .log import SwanLabSharedLog, Levels

__all__ = ["FONT", "SwanLabSharedLog", "Levels"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:42
@File: env.py
@IDE: pycharm
@Description:
    测试环境变量
"""
from swankit import env as E
from tutils import TEMP_DIR
import pytest
import shutil
import nanoid
import datetime
import os


def test_create_time():
    t = E.create_time()
    assert t.endswith("+00:00")
    d = datetime.datetime.fromisoformat(t)
    assert d.tzinfo == datetime.timezone.utc


def test_list_mode():
    ms = E.SwanLabMode.list()
    assert len(ms) == 4
    assert "disabled" in ms
    assert "cloud" in ms
    assert "backup" in ms
    assert "local" in ms


class TestGetFolder:
    """
    测试获取全局保存文件夹
    """

    def test_default(self):
        """
        默认情况
        """
        # 获取当前用户的主目录
        home = os.path.expanduser("~")
        folder = os.path.join(home, ".swanlab")
        if os.path.exists(folder):
            shutil.rmtree(folder)
        assert E.get_save_dir() == folder
        assert os.path.exists(folder)

    def test_env_abs(self):
        """
        设置了一个绝对路径的环境变量
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        os.environ[E.SwanLabSharedEnv.SWANLAB_FOLDER.value] = path
        assert not os.path.exists(path)
        assert E.get_save_dir() == path
        assert os.path.exists(path)

    def test_env_rel(self):
        """
        设置了一个相对路径的环境变量
        """
        abs_path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        rel_path = os.path.relpath(abs_path.__str__(), os.getcwd())
        os.environ[E.SwanLabSharedEnv.SWANLAB_FOLDER.value] = rel_path
        assert not os.path.exists(abs_path)
        assert E.get_save_dir() == abs_path
        assert os.path.exists(abs_path)

    def test_env_parent_not_exist(self):
        """
        父目录不存在
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10), "a")
        os.environ[E.SwanLabSharedEnv.SWANLAB_FOLDER.value] = path
        assert not os.path.exists(path)
        with pytest.raises(FileNotFoundError):
            E.get_save_dir()

    def test_env_not_a_folder(self):
        """
        文件夹不存在，但是文件存在
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        with open(path, "w") as f:
            f.write("test")
        os.environ[E.SwanLabSharedEnv.SWANLAB_FOLDER.value] = path
        assert os.path.exists(path)
        with pytest.raises(NotADirectoryError):
            E.get_save_dir()


class TestGetSwanlog:

    def test_default(self):
        """
        默认情况
        """
        pwd = os.getcwd()
        folder = os.path.join(pwd, "swanlog")
        if os.path.exists(folder):
            shutil.rmtree(folder)
        assert E.get_swanlog_dir() == folder
        assert not os.path.exists(folder)

    def test_env_abs(self):
        """
        设置了一个绝对路径的环境变量
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        os.environ[E.SwanLabSharedEnv.SWANLOG_FOLDER.value] = path
        if os.path.exists(path):
            shutil.rmtree(path)
        assert E.get_swanlog_dir() == path
        assert not os.path.exists(path)

    def test_env_rel(self):
        """
        设置了一个相对路径的环境变量
        """
        abs_path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        rel_path = os.path.relpath(abs_path.__str__(), os.getcwd())
        os.environ[E.SwanLabSharedEnv.SWANLOG_FOLDER.value] = rel_path
        if os.path.exists(abs_path):
            shutil.rmtree(abs_path)
        assert E.get_swanlog_dir() == abs_path
        assert not os.path.exists(abs_path)

    def test_env_parent_not_exist(self):
        """
        父目录不存在
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10), "a")
        os.environ[E.SwanLabSharedEnv.SWANLOG_FOLDER.value] = path
        assert not os.path.exists(path)
        with pytest.raises(FileNotFoundError):
            E.get_swanlog_dir()

    def test_env_not_a_folder(self):
        """
        文件夹不存在，但是文件存在
        """
        path = os.path.join(TEMP_DIR, nanoid.generate("1234567890abcdef", 10))
        with open(path, "w") as f:
            f.write("test")
        os.environ[E.SwanLabSharedEnv.SWANLOG_FOLDER.value] = path
        assert os.path.exists(path)
        with pytest.raises(NotADirectoryError):
            E.get_swanlog_dir()


class TestGetMode:
    """
    测试获取解析模式
    """

    def test_default(self):
        """
        默认情况
        """
        assert E.get_mode() == E.SwanLabMode.CLOUD.value

    def test_env(self):
        """
        设置了环境变量
        """
        os.environ[E.SwanLabSharedEnv.SWANLAB_MODE.value] = E.SwanLabMode.LOCAL.value
        assert E.get_mode() == E.SwanLabMode.LOCAL.value

    def test_unknown(self):
        """
        未知的模式
        """
        os.environ[E.SwanLabSharedEnv.SWANLAB_MODE.value] = "unknown"
        with pytest.raises(ValueError):
            E.get_mode()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/5 16:32
@File: key.py
@IDE: pycharm
@Description:
    与Key相关的回调函数触发时的模型
"""
from typing import Union, Optional, Dict, List, Literal, Tuple

from swankit.core import ChartType, ParseErrorInfo, MediaBuffer, ChartReference
from urllib.parse import quote
import os

ColumnClass = Literal["CUSTOM", "SYSTEM"]
SectionType = Literal["PINNED", "HIDDEN", "PUBLIC", "SYSTEM", "CUSTOM"]
YRange = Optional[Tuple[Optional[float], Optional[float]]]


class ColumnConfig:
    """
    列信息配置
    """

    def __init__(
        self,
        y_range: YRange = None,
        chart_name: Optional[str] = None,
        chart_index: Optional[str] = None,
        metric_name: Optional[str] = None,
        metric_color: Optional[Tuple[str, str]] = None,
    ):
        """
        生成的列信息配置对象
        :param y_range: y轴范围
        :param chart_name: 图表名称
        :param chart_index: 图表索引
        :param metric_name: 指标名称
        :param metric_color: 指标颜色
        """
        self.y_range: YRange = y_range
        self.chart_name: Optional[str] = chart_name
        self.chart_index: Optional[str] = chart_index
        self.metric_name: Optional[str] = metric_name
        self.metric_color: Optional[Tuple[str, str]] = metric_color

    def clone(
        self,
        y_range: YRange = None,
        chart_name: Optional[str] = None,
        chart_index: Optional[str] = None,
        metric_name: Optional[str] = None,
        metric_color: Optional[Tuple[str, str]] = None,
    ):
        """
        克隆一个新的ColumnConfig对象，并且可以修改其中的参数
        :param y_range: y轴范围
        :param chart_name: 图表名称
        :param chart_index: 图表索引
        :param metric_name: 指标名称
        :param metric_color: 指标颜色
        :return: 新的ColumnConfig对象
        """
        return ColumnConfig(
            y_range=y_range if y_range is not None else self.y_range,
            chart_name=chart_name if chart_name is not None else self.chart_name,
            metric_name=metric_name if metric_name is not None else self.metric_name,
            chart_index=chart_index if chart_index is not None else self.chart_index,
            metric_color=metric_color if metric_color is not None else self.metric_color,
        )


class ColumnInfo:
    """
    列信息，当创建列时，会生成这个对象
    """

    def __init__(
        self,
        key: str,
        kid: str,
        name: str,
        cls: ColumnClass,
        chart_type: ChartType,
        chart_reference: ChartReference,
        section_name: Optional[str],
        section_type: SectionType,
        section_sort: Optional[int] = None,
        error: Optional[ParseErrorInfo] = None,
        config: Optional[ColumnConfig] = None,
    ):
        """
        生成的列信息对象
        :param key: 生成的列名称，作为索引键值
        :param kid: 当前实验下，列的唯一id，与保存路径等信息有关，与云端请求无关
        :param name: 列的别名
        :param cls: 列的类型，CUSTOM为自定义列，SYSTEM为系统生成列
        :param chart_type: 列对应的图表类型
        :param chart_reference: 这个列对应图表的参考系，step为步数，time为时间
        :param section_name: 列的组名
        :param section_sort: 列在section中的参考排序，不代表实际排序（目前为冗余设计）
        :param error: 列的类型错误信息
        :param config: 列的额外配置信息
        """
        self.key = key
        self.kid = kid
        self.name = name
        self.cls = cls

        self.chart_type = chart_type
        self.chart_reference = chart_reference

        self.section_name = section_name
        self.section_sort = section_sort
        self.section_type = section_type

        self.error = error
        self.config = config

    @property
    def got(self):
        """
        传入的错误类型，如果列出错，返回错误类型，如果没出错，`暂时`返回None
        """
        if self.error is None:
            return None
        return self.error.got

    @property
    def expected(self):
        """
        期望的类型，如果列出错，返回期望的类型，如果没出错，`暂时`返回None
        """
        if self.error is None:
            return None
        return self.error.expected

    @property
    def key_encode(self):
        """
        对key进行url编码
        """
        return quote(self.key, safe="")


class MetricInfo:
    """
    指标信息，当新的指标数据被log时，会生成这个对象
    """

    __SUMMARY_NAME = "_summary.json"

    def __init__(
        self,
        column_info: ColumnInfo,
        metric: Optional[Dict],
        metric_buffers: Optional[List[MediaBuffer]],
        metric_summary: Optional[Dict],
        metric_step: Optional[int],
        metric_epoch: Optional[int],
        metric_file_name: Optional[str],
        swanlab_logdir: Optional[str],
        swanlab_media_dir: Optional[str],
        error: Optional[ParseErrorInfo] = None,
    ):
        """
        生成的指标信息对象
        :param column_info: 此指标对应的列信息
        :param metric: 此指标的数据
        :param metric_buffers: 此指标的媒体数据，如果为None，表示没有媒体数据
        :param metric_summary: 此指标的摘要信息
        :param metric_step: 此指标的步数
        :param metric_epoch: 此指标对应本地的行数
        :param metric_file_name: 此指标的文件名
        :param swanlab_logdir: swanlab在本次实验的log文件夹路径
        :param swanlab_media_dir: swanlab在本次实验的media文件夹路径
        :param error: 创建此指标时的错误信息
        """
        self.error = error
        self.column_info = column_info
        self.metric = metric
        self.metric_buffers = metric_buffers
        self.metric_summary = metric_summary
        self.metric_step = metric_step
        self.metric_epoch = metric_epoch
        _id = self.column_info.kid
        self.metric_file_path = None if self.is_error else os.path.join(swanlab_logdir, _id, metric_file_name)
        self.summary_file_path = None if self.is_error else os.path.join(swanlab_logdir, _id, self.__SUMMARY_NAME)
        self.swanlab_media_dir = swanlab_media_dir
        # 写入文件名称，对应上传时的文件名称：{key}/{文件名称}，文件夹名称为key
        if self.metric_buffers is not None:
            for i, buffer in enumerate(self.metric_buffers):
                buffer.file_name = "{}/{}".format(self.column_info.key_encode, metric["data"][i])

    @property
    def is_error(self) -> bool:
        """
        这条指标信息是否有错误，错误分几种：
            1. 列错误，列一开始就出现问题
            2. 重复错误
            3. 指标错误
        """
        return self.error is not None or self.column_error is not None

    @property
    def column_error(self) -> Optional[ParseErrorInfo]:
        """
        列错误信息
        """
        return self.column_info.error

    @property
    def data(self):
        """
        指标数据的data字段
        """
        if self.is_error:
            return None
        return self.metric["data"]

    def __str__(self):
        return str(self.data)

    def __repr__(self):
        return str(self.data)


class MetricErrorInfo(MetricInfo):
    def __init__(self, column_info: ColumnInfo, error: ParseErrorInfo):
        """
        错误的指标信息，简化输入参数
        :param column_info: 此指标对应的列信息
        :param error: 创建此指标时的错误信息
        """
        super().__init__(
            column_info,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            None,
            error,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/5 16:30
@File: error.py
@IDE: pycharm
@Description:
    错误模型
"""


class OperateErrorInfo:
    """
    操作错误信息，当操作员操作回调发生错误时，会生成这个对象，传给相应的回调函数
    """

    def __init__(self, desc: str):
        self.desc = desc
        """
        错误描述
        """

    def __str__(self):
        return f"SwanLabError: {self.desc}"



"""
@author: cunyue
@file: test_settings.py
@time: 2025/5/15 18:15
@description: 测试 settings
"""

import pytest


def test_lazy_settings():
    from swankit.core.settings import LazySettings

    settings = LazySettings()
    settings.exp_name = "test"
    settings.exp_colors = ("red", "blue")
    settings.description = "test description"

    assert settings.exp_name == "test"
    assert settings.exp_colors == ("red", "blue")
    assert settings.description == "test description"

    with pytest.raises(ValueError, match="exp_name can only be set once"):
        settings.exp_name = "test2"
    with pytest.raises(ValueError, match="exp_colors can only be set once"):
        settings.exp_colors = ("green", "yellow")
    with pytest.raises(ValueError, match="description can only be set once"):
        settings.description = "test description 2"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/15 13:23
@File: __init__.py
@IDE: pycharm
@Description:
    核心解析模块工具
"""
from .data import BaseType, MediaType, DataSuite, MediaBuffer, ParseResult, ParseErrorInfo, ChartReference
from .settings import SwanLabSharedSettings

ChartType = BaseType.Chart

__all__ = [
    "BaseType",
    "MediaType",
    "ChartType",
    "DataSuite",
    "MediaBuffer",
    "ParseResult",
    "ParseErrorInfo",
    "SwanLabSharedSettings",
    "ChartReference",
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 22:54
@File: __init__.py.py
@IDE: pycharm
@Description:
    swankit —— SwanLab Toolkit
    为 SwanLab 提供的工具包，包含了一些常用的工具函数、类、模块等
    为了语意方便，应该通过swankit的不同模块调用各个功能函数
"""

__all__ = ["env", "callback", "log"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/14 23:06
@File: __init__.py.py
@IDE: pycharm
@Description:
    回调类，规定回调函数的接口规范。
"""

from abc import ABC, abstractmethod
from .models import *
from swankit.core import SwanLabSharedSettings
from typing import Tuple, Optional


class SwanKitCallback(ABC):
    """
    SwanKitCallback，回调函数注册类，所有以`on_`和`before_`开头的函数都会在对应的时机被调用
    此处只定义会被调用的函数，用于接口规范
    """

    def on_init(self, proj_name: str, workspace: str, public: bool = None, logdir: str = None, *args, **kwargs):
        """
        执行`swanlab.init`时调用，此时运行时环境变量没有被设置，此时修改环境变量还是有效的
        :param logdir: str, 用户设置的日志目录
        :param proj_name: str, 项目名称
        :param public: bool, 是否为公开项目
        :param workspace: str, 工作空间
        :param kwargs: dict, 其他参数，为了增加灵活性，可以在on_init的时候设置一些其他类内参数
        """
        pass

    def before_run(self, settings: SwanLabSharedSettings, *args, **kwargs):
        """
        在运行实验之前调用
        :param settings: SwanLabSharedSettings, 运行时的共享配置
        """
        pass

    def before_init_experiment(
        self,
        run_id: str,
        exp_name: str,
        description: str,
        colors: Tuple[str, str],
        *args,
        **kwargs,
    ):
        """
        在初始化实验之前调用，此时SwanLabRun已经初始化完毕
        :param run_id: str, SwanLabRun的运行id
        :param exp_name: str, 实验名称
        :param description: str, 实验描述
        :param colors: Tuple[str, str], 实验颜色，[light, dark]
        """
        pass

    def on_run(self, *args, **kwargs):
        """
        SwanLabRun初始化完毕时调用
        """
        pass

    def on_run_error_from_operator(self, e: OperateErrorInfo, *args, **kwargs):
        """
        执行`on_run`错误时被操作员调用
        """
        pass

    def on_runtime_info_update(self, r: RuntimeInfo, *args, **kwargs):
        """
        运行时信息更新时调用
        :param r: RuntimeInfo, 运行时信息
        """
        pass

    def on_log(self, data: dict, step: Optional[int], *args, **kwargs):
        """
        每次执行swanlab.log时调用
        """
        pass

    def on_column_create(self, column_info: ColumnInfo, *args, **kwargs):
        """
        列创建回调函数,新增列信息时调用
        """
        pass

    def on_metric_create(self, metric_info: MetricInfo, *args, **kwargs):
        """
        指标创建回调函数,新增指标信息时调用
        """
        pass

    def on_stop(self, error: str = None, *args, **kwargs):
        """
        训练结束时的回调函数
        """
        pass

    @abstractmethod
    def __str__(self) -> str:
        """
        返回当前回调函数的名称，这条应该是一个全局唯一的标识
        在operator中会用到这个名称，必须唯一
        """
        pass


__all__ = ["SwanKitCallback", "ColumnInfo", "MetricInfo", "ColumnConfig", "OperateErrorInfo", "RuntimeInfo"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/17 16:32
@File: test_parse_docs.py
@IDE: pycharm
@Description:
    测试parse_docs.py
"""
import os
from scripts.parse_docs import parse, main
from tutils import TEMP_DIR
from unittest.mock import patch
import argparse
import pytest


@pytest.mark.parametrize("raw, expected", [
    (
            "Here is a link to [some document](/docs/wiki/some/path/to/document.md).",
            "Here is a link to [some document](./some/path/to/document)."
    ),
    (
            "And another link to [another document](/docs/wiki/another/path/to/document.md).",
            "And another link to [another document](./another/path/to/document)."
    ),
    (
            "No change to [normal link](https://example.com).",
            "No change to [normal link](https://example.com)."
    ),
    (
            "Multiple links [doc1](/docs/wiki/doc1.md) and [doc2](/docs/wiki/doc2.md).",
            "Multiple links [doc1](./doc1) and [doc2](./doc2)."
    ),
    (
            "Empty link []().",
            "Empty link []()."
    ),
    (
            "Link with no extension [doc](/docs/wiki/doc).",
            "Link with no extension [doc](/docs/wiki/doc)."
    ),
    (
            "Link with different extension [doc](/docs/wiki/doc.txt).",
            "Link with different extension [doc](/docs/wiki/doc.txt)."
    ),
])
def test_parse(raw, expected):
    """
    测试parse函数
    """
    assert parse(raw) == expected


def test_main():
    """
    测试main函数
    """
    path = os.path.join(TEMP_DIR, "test_parse_docs")
    os.makedirs(path)
    # ---------------------------------- 写入一些文件 ----------------------------------
    with open(os.path.join(path, "test1.md"), "w") as f:
        f.write("Here is a link to [some document](/docs/wiki/some/path/to/document.md).")
    with open(os.path.join(path, "test2.md"), "w") as f:
        f.write("And another link to [another document](/docs/wiki/another/path/to/document.md).")

    with patch("argparse.ArgumentParser.parse_args", return_value=argparse.Namespace(path=path)):
        main()

    with open(os.path.join(path, "test1.md"), "r") as f:
        assert f.read() == "Here is a link to [some document](./some/path/to/document)."
    with open(os.path.join(path, "test2.md"), "r") as f:
        assert f.read() == "And another link to [another document](./another/path/to/document)."
